/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Full Screen Mobile Add-on.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2011
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * * Matt Brubeck <mbrubeck@mozilla.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

"use strict";

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;

Cu.import("resource://gre/modules/Services.jsm");

const prefName = "app.orientation.default";
let gWin;

function isNativeUI() {
  let appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
  return (appInfo.ID == "{aa3c5121-dab2-40e2-81ca-7ea25febc110}");
}

let Screen = {
  set orientation(value) {
    if (value)
      Services.prefs.setCharPref(prefName, value);
    else if (Services.prefs.prefHasUserValue(prefName))
      Services.prefs.clearUserPref(prefName);
  },

  get orientation() {
    return Services.prefs.prefHasUserValue(prefName)
         ? Services.prefs.getCharPref(prefName) : "";
  }
};

let gMenuId = null;
let gStringBundle = null;

// This should be kept in sync with the values in locale/*/phony.properties
let uas = [
  ["Landscape", "Landscape"],
  ["Portrait", "Portrait"],
 ];

function init(aEvent) {
  if (this.selectedPanel.id != "prefs-container")
    return;

  let document = this.ownerDocument;
  let menu = document.getElementById("Screen-Orientation-menu");

  if (!menu) {
    // Create the menu.
    let setting = document.createElement("setting");
    setting.setAttribute("id", "Screen-Orientation-setting");
    setting.setAttribute("type", "control");
    setting.setAttribute("title", "Orientation");
    document.getElementById("prefs-list").insertBefore(setting, document.getElementById("prefs-sync"));

    menu = document.createElement("menulist");
    menu.setAttribute("id", "Screen-Orientation-menu");
    setting.appendChild(menu);
  }

  // At this point (on Android devices only?), appending items to the menu does
  // not work.  It seems the XBL binding is not always available immediately,
  // so we need a timeout before using it.
  gWin.setTimeout(function() {
    if (!menu.firstChild || !menu.firstChild.hasChildNodes) {
      for (let i=0; i<uas.length; i++) {
        let [label, value] = uas[i];
        menu.appendItem(label, value); // XXX not available until XBL kicks in
      }
      menu.addEventListener("select", function() Screen.orientation= menu.selectedItem.value, false);
    }

    let item = menu.getElementsByAttribute("value", Screen.orientation)[0];
    if (!item)
      item = menu.appendItem("Custom", Screen.orientation);
    menu.selectedItem = item;
  }, 0);
}

function initUAList() {
  if (!gStringBundle)
    gStringBundle = Services.strings.createBundle("chrome://Screen/locale/Screen.properties");

  let uaItemLabel, uaItemValue = null;

  for (let i=0; i<uas.length; i++) {
    try {
      uaItemLabel = gStringBundle.GetStringFromName("Screen.uas." + i);
      uaItemValue = gStringBundle.GetStringFromName("Screen.uas." + i + ".value");
      if (uaItemLabel) {
        uas[i] = [uaItemLabel, uaItemValue];
      }
    } catch (e) { Cu.reportError(e); }
  }
}

function selectUA() {
  let labels = [];
  let values = [];
  let found = false;

  for (let i = 0; i < uas.length; i++) {
    let [label, value] = uas[i];
    if (value == Screen.orientation) {
      found = true;
      labels.unshift(label);
      values.unshift(value);
    } else {
      labels.push(label);
      values.push(value);
    }
  }

  if (!found) {
    labels.push("Custom");
    values.push(Screen.orientation);
  }

  let res = { value: 0 };
  if (!gStringBundle)
    gStringBundle = Services.strings.createBundle("chrome://Screen/locale/Screen.properties");
  let selectTitleString = gStringBundle.GetStringFromName("Screen.name");
  let selectMsgString = gStringBundle.GetStringFromName("Screen.selectMsg");
  if (Services.prompt.select(null, selectTitleString, selectMsgString, labels.length, labels, res))
    Screen.orientation = values[res.value];
}

function load(win) {
  initUAList();

  gWin = win;
  if (isNativeUI()) {
    gMenuId = win.NativeWindow.menu.add("Screen", null, selectUA);
  } else {
    let panels = win.document.getElementById("panel-items");
    panels.addEventListener("select", init, false);
  }
}

function unload(win) {
  if (isNativeUI()) {
    win.NativeWindow.menu.remove(gMenuId);
  } else {
    let setting = win.document.getElementById("Screen-Orientation-setting");
    if (setting)
      setting.parentNode.removeChild(setting);
    win.document.getElementById("panel-items").removeEventListener("select", init, false);
  }
  Screen.orientation = null;
}

var listener = {
  onOpenWindow: function(aWindow) {
    // Wait for the window to finish loading
    let win = aWindow.QueryInterface(Ci.nsIInterfaceRequestor)
                     .getInterface(Ci.nsIDOMWindowInternal || Ci.nsIDOMWindow);
    win.addEventListener("UIReady", function onReady(aEvent) {
      win.removeEventListener("UIReady", onReady, false);
      load(win);
    }, false);
  },

  // Unused:
  onCloseWindow: function(aWindow) { },
  onWindowTitleChange: function(aWindow, aTitle) { }
};

/* Bootstrap Interface */

function startup(aData, aReason) {
  // Load in existing windows.
  let enumerator = Services.wm.getEnumerator("navigator:browser");
  while(enumerator.hasMoreElements()) {
    let win = enumerator.getNext();
    load(win);
  }

  // Load in future windows.
  Services.wm.addListener(listener);
}

function shutdown(aData, aReason) {
  if (aReason == APP_SHUTDOWN)
    return;

  Services.wm.removeListener(listener);

  let enumerator = Services.wm.getEnumerator("navigator:browser");
  while(enumerator.hasMoreElements()) {
    let win = enumerator.getNext();
    unload(win);
  }
}

function install(aData, aReason) {}
function uninstall(aData, aReason) {}
